// This header file defines interrupts and routines that occur once per rev.

// PLIB
#define _SUPPRESS_PLIB_WARNING 1
#include "plib.h"

// For interrupts
#include <xc.h>
#include <sys/attribs.h>

volatile unsigned long capture_time = 0x00FF;
volatile unsigned long this_capture = 0;
volatile unsigned char lo_rpm = 0;
volatile unsigned long cap_samples[3] = {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF};

void initCycleCapture(){
  // Set up Timer 23. The period is the "LOW RPM" threshold.
  OpenTimer23(T2_ON | T23_PS_1_1 | T23_SOURCE_INT, 0x02FFFFFF);
  
  // Set up input capture 1
  OpenCapture1(IC_EVERY_RISE_EDGE | IC_INT_1CAPTURE | IC_CAP_32BIT | IC_ON );
  ConfigIntCapture1(IC_INT_ON | IC_INT_PRIOR_2 | IC_INT_SUB_PRIOR_2 );
  INTClearFlag(INT_IC1);
  PPSInput(3, IC1, RPB13);
  mPORTBSetPinsDigitalIn(BIT_13);    //Set port as input
  
  // Also set up timer overflow interrupt to catch "timeout" condition
  ConfigIntTimer23(T23_INT_ON | T23_INT_PRIOR_2);
  mT23ClearIntFlag();
}

// Retrieves the median of three samples - simple median filtration.
// Based on http://embeddedgurus.com/stack-overflow/2010/10/median-filtering/
unsigned long med3(unsigned long a, unsigned long b, unsigned long c)
{
    if ((a <= b) && (a <= c)) {
        return (b <= c) ? b : c;
    } else if ((b <= a) && (b <= c)) {
        return (a <= c) ? a : c;
    } else {
        return (a <= b) ? a : b;
    }
}

// Timer 2 "overflow" indicating low or no RPM.
void __ISR(_TIMER_23_VECTOR, IPL2AUTO) Timer23Handler(void)
{
    lo_rpm = 1;
    rpm = 0;
    phs_x = 0;
    phs_y = 0;
    amp_x = 0;
    amp_y = 0;
    mT23ClearIntFlag();
}

// === IC1 ISR =================================================
// Just grabs what the input capture peripheral saw on Timer2.
// Vector, interrupt priority level, handler
void __ISR(_INPUT_CAPTURE_1_VECTOR, IPL2AUTO) IC1Handler(void) 
{ 
    // Obtain capture and restart Timer2
    this_capture = mIC1ReadCapture();
    WriteTimer23(0);
    // If we didn't overflow the timer, measure the RPM
    if (lo_rpm == 0){
        // Load the new sample up
        cap_samples[2] = cap_samples[1];
        cap_samples[1] = cap_samples[0];
        cap_samples[0] = this_capture;
        capture_time = med3(cap_samples[0], cap_samples[1], cap_samples[2]);
        
        // Record RPM
        rpm = 2400000000.0/capture_time;
        if (rpm > 9999) rpm = 9999;
        
        // Update measurements from sample data
        phs_x = phs_meas_x;
        phs_y = phs_meas_y;
        if (amp_meas_x < 9999){
            amp_x = (float)amp_meas_x * (s_x / 70.0) * CAL_X;
        }
        if (amp_meas_y < 9999){
            amp_y = (float)amp_meas_y * (s_y / 70.0) * CAL_Y;
        }
        amp_meas_x = 0;
        amp_meas_y = 0;
        
        // If we peaked a channel, record the new peak.
        if (amp_x > p_amp_x) {
            p_rpm_x = rpm;
            p_amp_x = amp_x;
            p_phs_x = phs_x;
        }
        if (amp_y > p_amp_y) {
            p_rpm_y = rpm;
            p_amp_y = amp_y;
            p_phs_y = phs_y;
        } 
        
        // If the next cycle is to be a null vector record cycle, flag it
        if (start_x_null == 1){
            start_x_null = 0;
            getting_x_null = 1;
        }
        if (start_y_null == 1){
            start_y_null = 0;
            getting_y_null = 1;
        }
        
        // If the last cycle was the null vector record cycle, end record
        if (getting_x_null == 1){
            getting_x_null = 0;
            x_null = 1;
        }
        if (getting_y_null == 1){
            getting_y_null = 0;
            y_null = 1;
        }
    }
    lo_rpm = 0;
    mIC1ClearIntFlag();     // Clear the interrupt flag
}

