// This header file defines interrupts and routines that occur once per sample.

// ADC Control Bytes
#define ADC_CONV         0b11110001 // +-10V, normal operation, internal ref
#define TWOS_COMP_NEG_SE 0xFFFFF000 //sign extend negative numbers from 12 bit to 32 bit

volatile int adc_read;

void updateReading(unsigned char chan){
    int read1; // Two's complement char b/c in bipoloar mode -> ignore
    int read2; // Bits31-7 0's, bits6-0 MSB-B5
    int read3; // Bits31-8 0's, bits7-3 B4-B0, bits2-0 0's 
    unsigned int sign; // MSB of 12-bit conversion
    
    if (chan == 1){
        PORTSetBits(IOPORT_B, BIT_4);// Pulls ADC 1 CS low
    } else if (chan == 2){
        PORTSetBits(IOPORT_A, BIT_3);// Pulls ADC 2 CS low
    }
    
    while(!SPI1STATbits.SPITBE);    // wait for TX buffer to empty
    WriteSPI1(ADC_CONV);            // write control byte to ADC chip
    while(!SPI1STATbits.SPITBE);    // wait for TX buffer to empty 
    while(!SPI1STATbits.SPIRBF);	// wait for RX buffer to fill 
    read1 = ReadSPI1();             // read RX buffer 
   
    WriteSPI1(0x00);                // Force CLK to run for RX transfer
    while(!SPI1STATbits.SPITBE);    // wait for TX buffer to empty 
    while(!SPI1STATbits.SPIRBF);	// wait for RX buffer to fill 
    read2 = ReadSPI1();             // read RX buffer 
    
    WriteSPI1(0x00);                // Force CLK to run for RX transfer
    while(!SPI1STATbits.SPITBE);    // wait for TX buffer to empty 
    while(!SPI1STATbits.SPIRBF);	// wait for RX buffer to fill 
    read3 = ReadSPI1();             // read RX buffer 

    PORTClearBits(IOPORT_B, BIT_4);   // Pull CS high to end transfer
    PORTClearBits(IOPORT_A, BIT_3);
   
    //Convert 2 bytes into 12-bit read
    sign = ((read2<<25) & 0x80000000)>>31; // kill all bits except msb
    if (sign == 1){
        adc_read = TWOS_COMP_NEG_SE | (((read2<<8)|read3)>>3); // sign extend for two's comp
    } 
    else if (sign == 0){
        adc_read = ((read2<<8)|read3)>>3; // MSB is 0 positive twos comp so regular binary 
    }
    
    // Wave was inverted while conditioned
    adc_read = -adc_read;
}

// Set up to sample in overflow interrupts at rate MEASURE_F
void initWaveSampler(void){
  // Setup a timer overflow interrupt to take periodic samples
  OpenTimer1(T1_ON  | T1_SOURCE_INT | T1_PS_1_1 , pb_clock/MEASURE_F);
  ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_7);
  mT1ClearIntFlag();
  
  // Set up SPI Channel 2 for the MAX1272's
  PPSInput(2, SDI1, RPB5);      //SPI 1 input receives information (pin 9)
  PPSOutput(3, RPA4, SDO1);     //SPI 1 output sends control byte
  // 3 consecutive 8 bit reads necessary to get 12-bit value
  // SCK = 40/32 MHz ~ 625 KHz, ADC has max threshold of 1.4MHz
  // idle low, active-high
  SpiChnOpen(1, SPI_OPEN_MODE8 | SPI_OPEN_MSTEN |SPI_OPEN_CKE_REV|
          SPI_OPEN_ON, 32);
  
  // Set up CS pin for SPI ADC X    
  PORTSetPinsDigitalOut(IOPORT_B, BIT_4); //Sets RPB4 for as output pin
  PORTClearBits(IOPORT_B, BIT_4); //sets it high initially b/c CS is active low
  
  // Set up CS pin for SPI ADC Y
  PORTSetPinsDigitalOut(IOPORT_A, BIT_3);
  PORTClearBits(IOPORT_A, BIT_3);
}

volatile float cur_phs = 0;
volatile int adjust_read = 0;
volatile unsigned int ind = 0;

// Timer 1 interrupt handler: waveform sampling, peak detection.
void __ISR(_TIMER_1_VECTOR, IPL7AUTO) Timer1Handler(void)
{
  // Clear interrupt flag
  mT1ClearIntFlag();
  
  // Calculate the phase
  cur_phs = (rpm*ReadTimer23())/(6666667);
  if (cur_phs > 360) cur_phs = 360;
  ind = (unsigned int)(cur_phs*2);
  
  // Make measurements, looking for where wave peak occurs.
  updateReading(1);
  if (x_null == 1){
      adjust_read = adc_read - null_arr_x[ind];
  } else {
      adjust_read = adc_read;
  }
  
  if (adjust_read > amp_meas_x && ((adjust_read-amp_meas_x) < PEAK_JUMP || amp_meas_x == 0)){
      amp_meas_x = adjust_read;
      phs_meas_x = cur_phs - 45;
      // The 60 offset shifts the phase into a "nice" region of the plot.
  }
  
  // If we're in the process of measuring a null vector, store the read.
  if (getting_x_null == 1){
      null_arr_x[ind] = adc_read;
  }
  
  updateReading(2);
  if (y_null == 1){
      adjust_read = adc_read - null_arr_y[ind];
  } else {
      adjust_read = adc_read;
  }
  
  if (adjust_read > amp_meas_y && ((adjust_read-amp_meas_y) < PEAK_JUMP || amp_meas_y == 0)){
      amp_meas_y = adjust_read;
      phs_meas_y = cur_phs - 45;
  }
  
  if (getting_y_null == 1){
      null_arr_y[ind] = adc_read;
  }
}
